"""
B-ASIC Graph ID module for handling IDs of different objects in a graph.
TODO: More info
"""

from collections import defaultdict
from typing import NewType, Union, DefaultDict

GraphIDType = NewType("GraphIDType", str)
GraphIDNumber = NewType("GraphIDNumber", int)

class GraphIDGenerator:
    """
    A class that generates Graph IDs for objects. 
    """

    _next_id_number: DefaultDict(GraphIDType, GraphIDNumber)

    def __init__(self):
        self._next_id_number = defaultdict(lambda: 1)       # Initalises every key element to 1

    def get_next_id(self, graph_id_type: GraphIDType):
        """
        Retrns the next graph id for a certain graph id type.
        """
        graph_id = GraphID(graph_id_type, self._next_id_number[graph_id_type])
        self._next_id_number[graph_id_type] += 1            # Increase the current id number
        return graph_id


class GraphID:
    """
    Graph ID class that saves the id of for a graph object.

    The ID consists of an id_type that is saved as a string and an 
    id_number that is saved as an integer. 
    """

    graph_id_type: GraphIDType
    graph_id_number: GraphIDNumber


    def __init__(self, graph_id_type: GraphIDType, graph_id_number: GraphIDNumber):
        self.graph_id_type = graph_id_type
        self.graph_id_number = graph_id_number


    def __str__(self) -> str:
        return graph_id_type + str(graph_id_number)


    def __repr__(self) -> str:
        return str(self)


    def __hash__(self) -> int:
        return hash(str(self))


    def __eq__(self, other: GraphID) -> bool:
        return self.graph_id_type == other.graph_id_type and \
            self.graph_id_number == other.graph_id_number

    
    def get_next_id(self) -> GraphID:
        """
        Returns a new GraphID of the same type with an incremented id number.
        """ 
        return GraphID(self.graph_id_type, self.graph_id_number + 1)
